#ifndef XWINDOWGRAPHICSCONTEXT_H_
#define XWINDOWGRAPHICSCONTEXT_H_

#include <GraphicsContext.h>

#include <X11/Xlib.h>
#include <GL/glx.h>
#include <GL/gl.h>

#include <ThreadPrimitives.h>

#include "OpenGLCommandServer.h"

struct XExtendedGLFuncs : public ExtendedGLFuncs
{
	XExtendedGLFuncs();
	virtual ~XExtendedGLFuncs();
};

class XWindowGraphicsContext : public GraphicsContext
{
	GLXWindow m_glxWin;
	GLXFBConfig m_config;

	Display* m_disp;

	Mutex m_monitorLock;

	OpenGLCommandServer* m_glServer;

	GLXContext m_context;

	void ServerRun();

	void SwapBuffers();

	public:
	XWindowGraphicsContext(Window win, const GLXFBConfig& config, int width, int height);
	~XWindowGraphicsContext();

	virtual void ResizeScreen(int width, int height) {m_glServer->ResizeScreen(width, height);}
	virtual void StartFrame() {m_glServer->StartFrame();}
	virtual void SubmitRenderCall(RenderCall func, const Matrix<4,4>& modelView, float alpha=1.1f)
	{ m_glServer->SubmitRenderCall(func, modelView, alpha);}
	virtual void RunFunction(const boost::function<void ()>& func) {m_glServer->RunFunction(func);}
	virtual void EndFrame() {m_glServer->EndFrame();}

	virtual void FlushWait() {m_glServer->FlushWait();}

	virtual Matrix<4,4> GetProjectionMatrix() {return m_glServer->GetProjectionMatrix();}

	virtual void SetNearPlane(float n) {m_glServer->SetNearPlane(n);}
	virtual float GetNearPlane() {return m_glServer->GetNearPlane();}

	virtual void SetFarPlane(float f) {m_glServer->SetFarPlane(f);}
	virtual float GetFarPlane() {return m_glServer->GetFarPlane();}

	virtual void SetCurrentContext();
};

#endif
