#ifndef WIN32GRAPHICSCONTEXT_H_
#define WIN32GRAPHICSCONTEXT_H_

#include "GraphicsContext.h"
#include "ThreadPrimitives.h"
#include "Algebra.h"
#include "BoundedQueue.h"
#include "OpenGLRenderContext.h"

#include <windows.h>

#include <set>

#include <boost/function.hpp>

class Win32GraphicsContext : public GraphicsContext
{
	HWND m_hwnd;

	HDC m_dc;
	HGLRC m_rc;

	Mutex m_monitorLock;

	OpenGLRenderContext m_glContext;

	Thread* m_thread;
	CommandQueue* m_commandQueue;
	
	Mutex m_initDoneMutex;
	Condition m_initDoneCond;

	int m_width;
	int m_height;

	void Run();


	struct RenderFrameCommand : public CommandQueue::Command
	{
		Win32GraphicsContext& handler;

		GraphicsFrame& frame;
		GraphicsContext::RenderOptions options;
		GraphicsFrame::Freeze freeze;

		RenderFrameCommand(Win32GraphicsContext& pHandler, GraphicsFrame& pFrame, const GraphicsContext::RenderOptions& pOptions, GraphicsFrame::Freeze& pFreeze)
			: handler(pHandler)
			, frame(pFrame)
			, options(pOptions)
			, freeze(pFreeze)
		{};

		virtual void ProcessCommand() {handler.ProcessCommand(*this);}
	};
	void ProcessCommand(RenderFrameCommand& cmd);


	struct ResizeViewportCommand : public CommandQueue::Command
	{
		Win32GraphicsContext& handler;

		int width;
		int height;

		ResizeViewportCommand(Win32GraphicsContext& pHandler, int pWidth, int pHeight)
			: handler(pHandler)
			, width(pWidth), height(pHeight)
		{};

		virtual void ProcessCommand() {handler.ProcessCommand(*this);}
	};
	void ProcessCommand(ResizeViewportCommand& cmd);

public:
	Win32GraphicsContext(HWND hwnd);
	~Win32GraphicsContext();

	virtual void ResizeScreen(int width, int height);

	virtual void RenderGraphicsFrame(GraphicsFrame& frame, const GraphicsContext::RenderOptions& options);
	virtual void FlushWait();

	virtual int GetWidth();
	virtual int GetHeight();
};

#endif
