#ifndef BOUNDEDQUEUE_H_
#define BOUNDEDQUEUE_H_

#include "ThreadPrimitives.h"
#include "Utils.h"

#include <list>

template <class T>
class BoundedQueue
{
	Mutex m_mutex;
	Condition m_notEmptyCond;
	Condition m_notFullCond;
	Condition m_emptyCond;

	typedef std::list<T> ListType;
	ListType m_queue;

	const int m_maxMessages;
	int m_numMessages;

	public:
	BoundedQueue(int maxMessages)
	: m_maxMessages(maxMessages)
	, m_numMessages(0)
	{
		ASSERT(m_maxMessages > 0);
	}

	~BoundedQueue()
	{
		ASSERT(m_queue.empty());
	}

	int GetSize()
	{
		Mutex::ScopedLock monitorLock(m_mutex);

		return m_numMessages;
	}

	void WaitEmpty()
	{
		Mutex::ScopedLock monitorLock(m_mutex);

		if(m_numMessages > 0)
		{
			m_emptyCond.Wait(m_mutex);
		}
	}

	void Push(const T& msg)
	{
		Mutex::ScopedLock monitorLock(m_mutex);

		while(m_numMessages == m_maxMessages)
		{
			m_notFullCond.Wait(m_mutex);
		}
		ASSERT(m_numMessages < m_maxMessages);

		++m_numMessages;
		
		m_queue.push_front(msg);

		if(m_numMessages == 1)
		{
			m_notEmptyCond.SignalAll();
		}
	}

	T Pop()
	{
		Mutex::ScopedLock monitorLock(m_mutex);

		while(m_numMessages == 0)
		{
			m_notEmptyCond.Wait(m_mutex);
		}
		ASSERT(m_numMessages > 0);

		--m_numMessages;

		T ret = m_queue.back();
		m_queue.pop_back();

		if(m_numMessages == m_maxMessages-1)
		{
			m_notFullCond.SignalAll();
		}

		if(m_numMessages == 0)
		{
			m_emptyCond.SignalAll();
		}

		return ret;		
	}

};

#endif

