#ifndef SDLWINDOW_H_
#define SDLWINDOW_H_

#include <GenericWindow.h>

#include <GraphicsContext.h>
#include <SDLWindowKeyboardInput.h>
#include <MouseInput.h>
#include <OpenGLRenderContext.h>

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include <string>

class SDLWindowMouseInput : public MouseInput
{
	friend class SDLWindow;
	void PollMouse();
};

class SDLWindowGraphicsContext : public GraphicsContext
{
	OpenGLRenderContext m_glContext;

	int m_viewportWidth;
	int m_viewportHeight;

	float m_near;
	float m_far;
	Matrix<4,4> MakeProjectionMatrix(float fov, float aspectRatio, float near, float far) const;

	public:
	SDLWindowGraphicsContext(int width, int height);
	~SDLWindowGraphicsContext();

	virtual void ResizeScreen(int width, int height);
	
	virtual void RenderGraphicsFrame(GraphicsFrame& frame);
	virtual void FlushWait() {}

	virtual Matrix<4,4> GetProjectionMatrix();

	virtual void SetNearPlane(float n) {m_near = n;}
	virtual float GetNearPlane() {return m_near;}

	virtual void SetFarPlane(float f) {m_far = f;}
	virtual float GetFarPlane() {return m_far;}

	virtual void SetCurrentContext() {}
};

class SDLWindow : public GenericWindow
{
	std::string m_title;

	int m_width;
	int m_height;

	SDL_Surface* m_screen;

	SDLWindowKeyboardInput m_keyboardInput;
	SDLWindowMouseInput m_mouseInput;
	SDLWindowGraphicsContext* m_graphicsContext;


	public:
	class Exception
	{
		public:
		enum ExceptionID
		{
			EID_FATAL,
		};

		private:
		ExceptionID m_id;
		std::string m_msg;

		public:
		Exception(ExceptionID id, const std::string& msg)
			: m_id(id)
			, m_msg(msg)
		{}

		ExceptionID GetID() const {return m_id;}
		const std::string GetMessage() const {return m_msg;}
	};

	SDLWindow(const std::string& title, int width, int height);
	virtual ~SDLWindow();

	void PumpEvents();

	virtual GraphicsContext& GetGraphicsContext();
	virtual KeyboardInput& GetKeyboardInput() {return m_keyboardInput;}
	virtual MouseInput& GetMouseInput();
};

#endif
