#ifndef SAFEPOINTERLIST_H_
#define SAFEPOINTERLIST_H_

// Provides a list of pointers that will delete its contents
// when destructed

#include <vector>

template <class T>
struct SafePointerList
{
	std::vector<T*> objs;

	~SafePointerList() // Delete the objects in objs
	{
		for(std::vector<T*>::iterator iter = objs.begin(); iter != objs.end(); ++iter)
		{
			delete *iter;
		}
	}

	void ClearRelease() {objs.clear();} // Clears the vector, effectively releasing control of the objects contained from being freed
	void TakeFrom(SafePointerList<T*>& other)
	{
		for(std::vector<T*>::iterator iter = other.objs.begin(); iter != other.objs.end(); ++iter)
		{
			objs.push_back(*iter);
		}

		other.ClearRelease();
	}
};

#endif
