#include "SDLWindowKeyboardInput.h"

static KeyboardInput::KeyType SDLKeyToKeyType(SDLKey key)
{
	switch(key)
	{
		case SDLK_ESCAPE: return KeyboardInput::KT_ESC;
		case SDLK_F1: return KeyboardInput::KT_F1;
		case SDLK_F2: return KeyboardInput::KT_F2;
		case SDLK_F3: return KeyboardInput::KT_F3;
		case SDLK_F4: return KeyboardInput::KT_F4;
		case SDLK_F5: return KeyboardInput::KT_F5;
		case SDLK_F6: return KeyboardInput::KT_F6;
		case SDLK_F7: return KeyboardInput::KT_F7;
		case SDLK_F8: return KeyboardInput::KT_F8;
		case SDLK_F9: return KeyboardInput::KT_F9;
		case SDLK_F10: return KeyboardInput::KT_F10;
		case SDLK_F11: return KeyboardInput::KT_F11;
		case SDLK_F12: return KeyboardInput::KT_F12;
		case SDLK_PAUSE: return KeyboardInput::KT_PAUSE;
		
		case SDLK_BACKQUOTE: return KeyboardInput::KT_TILDE;
		case SDLK_1: return KeyboardInput::KT_1;
		case SDLK_2: return KeyboardInput::KT_2;
		case SDLK_3: return KeyboardInput::KT_3;
		case SDLK_4: return KeyboardInput::KT_4;
		case SDLK_5: return KeyboardInput::KT_5;
		case SDLK_6: return KeyboardInput::KT_6;
		case SDLK_7: return KeyboardInput::KT_7;
		case SDLK_8: return KeyboardInput::KT_8;
		case SDLK_9: return KeyboardInput::KT_9;
		case SDLK_0: return KeyboardInput::KT_0;
		case SDLK_MINUS: return KeyboardInput::KT_MINUS;
		case SDLK_PLUS: return KeyboardInput::KT_EQUAL;
		case SDLK_BACKSPACE: return KeyboardInput::KT_BACKSPACE;
		
		case SDLK_TAB: return KeyboardInput::KT_TAB;
		case SDLK_q: return KeyboardInput::KT_Q;
		case SDLK_w: return KeyboardInput::KT_W;
		case SDLK_e: return KeyboardInput::KT_E;
		case SDLK_r: return KeyboardInput::KT_R;
		case SDLK_t: return KeyboardInput::KT_T;
		case SDLK_y: return KeyboardInput::KT_Y;
		case SDLK_u: return KeyboardInput::KT_U;
		case SDLK_i: return KeyboardInput::KT_I;
		case SDLK_o: return KeyboardInput::KT_O;
		case SDLK_p: return KeyboardInput::KT_P;
		case SDLK_LEFTBRACKET: return KeyboardInput::KT_SQUARE_BRACKET_L;
		case SDLK_RIGHTBRACKET: return KeyboardInput::KT_SQUARE_BRACKET_R;
		case SDLK_BACKSLASH: return KeyboardInput::KT_BACKSLASH;
		
		case SDLK_CAPSLOCK: return KeyboardInput::KT_CAPSLOCK;
		case SDLK_a: return KeyboardInput::KT_A;
		case SDLK_s: return KeyboardInput::KT_S;
		case SDLK_d: return KeyboardInput::KT_D;
		case SDLK_f: return KeyboardInput::KT_F;
		case SDLK_g: return KeyboardInput::KT_G;
		case SDLK_h: return KeyboardInput::KT_H;
		case SDLK_j: return KeyboardInput::KT_J;
		case SDLK_k: return KeyboardInput::KT_K;
		case SDLK_l: return KeyboardInput::KT_L;
		case SDLK_SEMICOLON: return KeyboardInput::KT_SEMICOLON;
		case SDLK_QUOTEDBL: return KeyboardInput::KT_APOSTROPHE;
		case SDLK_RETURN: return KeyboardInput::KT_ENTER;

		case SDLK_LSHIFT: return KeyboardInput::KT_SHIFT;
		case SDLK_RSHIFT: return KeyboardInput::KT_SHIFT;
		case SDLK_z: return KeyboardInput::KT_Z;
		case SDLK_x: return KeyboardInput::KT_X;
		case SDLK_c: return KeyboardInput::KT_C;
		case SDLK_v: return KeyboardInput::KT_V;
		case SDLK_b: return KeyboardInput::KT_B;
		case SDLK_n: return KeyboardInput::KT_N;
		case SDLK_m: return KeyboardInput::KT_M;
		case SDLK_COMMA: return KeyboardInput::KT_COMMA;
		case SDLK_PERIOD: return KeyboardInput::KT_PERIOD;
		case SDLK_SLASH: return KeyboardInput::KT_SLASH;

		case SDLK_RCTRL: return KeyboardInput::KT_CTRL;
		case SDLK_LCTRL: return KeyboardInput::KT_CTRL;
		case SDLK_SPACE: return KeyboardInput::KT_SPACE;
		
		case SDLK_LEFT: return KeyboardInput::KT_LEFT;
		case SDLK_RIGHT: return KeyboardInput::KT_RIGHT;
		case SDLK_UP: return KeyboardInput::KT_UP;
		case SDLK_DOWN: return KeyboardInput::KT_DOWN;

		case SDLK_INSERT: return KeyboardInput::KT_INSERT;
		case SDLK_DELETE: return KeyboardInput::KT_DELETE;
		case SDLK_HOME: return KeyboardInput::KT_HOME;
		case SDLK_END: return KeyboardInput::KT_END;
		case SDLK_PAGEUP: return KeyboardInput::KT_PG_UP;
		case SDLK_PAGEDOWN: return KeyboardInput::KT_PG_DOWN;

		default: return KeyboardInput::KT_INVALID;
	};
}

void SDLWindowKeyboardInput::PostKeyDownEvent(SDLKey key)
{
	KeyboardInput::KeyEvent nextEvent;
	nextEvent.key = SDLKeyToKeyType(key);
	nextEvent.pressed = true;

	PostKeyPress(nextEvent);	
}

void SDLWindowKeyboardInput::PostKeyUpEvent(SDLKey key)
{
	KeyboardInput::KeyEvent nextEvent;
	nextEvent.key = SDLKeyToKeyType(key);
	nextEvent.pressed = false;

	PostKeyPress(nextEvent);	
}

