#include <FileFetcher.h>

#include <boost/bind.hpp>

#include <iostream>

const int LOAD_ATTEMPTS = 50;
const int MAX_SLEEP_TIME = 3; // in seconds
// Maximum time this test will take to complete is LOAD_ATTEMPTS * MAX_SLEEP_TIME

static void FetchDoneCB(FileFetcher::FileLoad* load)
{
	load->Error();

	ASSERT(load->Done());
}

void TestThread(FileFetcher* fetcher, int threadId)
{
	srand(threadId);

	// Loop LOAD_ATTEMPTS time, and for each time, load a file, wait a random amount of time
	// up to MAX_SLEEP_TIME, then destroy the file.
	// If the file completed, check the contents (if we can)
	for(int i=0; i<LOAD_ATTEMPTS; ++i)
	{
		const int NUM_DIFFERENT_FILES = 2;
		int fileNum = rand() % NUM_DIFFERENT_FILES;

		const char* files[] =
		{
			"test1.txt",
			"test2.txt",
		};
		FileFetcher::FileLoad testLoad(*fetcher, ResourceIdentifier(files[fileNum]), &FetchDoneCB);

		const int MAX_SLEEP_TIME_IN_MS = MAX_SLEEP_TIME * 1000;
		Sleep(rand()%(MAX_SLEEP_TIME_IN_MS/2));

		if(rand()%2 == 0)
		{
			// Randomly stop the load processing before we destroy the object
			testLoad.StopLoadProcess();
		}

		Sleep(rand()%(MAX_SLEEP_TIME_IN_MS/2));

		std::cout << "(" << threadId << ", " << i << ") File load status for \"" << files[fileNum] << "\":  ";
		if(testLoad.Done())
		{
			if(testLoad.Error())
			{
				std::cout << "Error reading file in test loader" << std::endl;
			}
			else
			{
				std::cout << "File load successful" << std::endl;

				const char* fileTestStrings[] =
				{
					"Hello this is a test",
					"I am test number 2.  Pleased to meet you!!!!",
				};

				const int TEST_PREFIX = 10;

				char loadedData[TEST_PREFIX];
				memcpy(loadedData, testLoad.GetData(), TEST_PREFIX);
		
				char testData[TEST_PREFIX];
				memcpy(testData, fileTestStrings[fileNum], TEST_PREFIX);

				// Test that the first 10 characters are correct
				for(int i=0; i<10; ++i)
				{
					ASSERT(loadedData[i] == testData[i]);
				}
			}
		}
		else
		{
			std::cout << "Did not finish loading file" << std::endl;
		}
	}
}

int main(int argc, char** args)
{
	FileFetcher fetcher(20000000);

	const int NUM_LOADER_THREADS = 100;
	Thread* threads[NUM_LOADER_THREADS];
	for(int i=0; i<NUM_LOADER_THREADS; ++i)
	{
		threads[i] = new Thread(boost::bind(&TestThread, &fetcher, i));
	}

	for(int i=0; i<NUM_LOADER_THREADS; ++i)
	{
		delete threads[i];
	}

	std::cout << "Done." << std::endl;

	return 0;
}
