#include "PathRelativeTo.h"
#include <Utils.h>

boost::filesystem::path PathRelativeTo(const boost::filesystem::path& relativeTo, const boost::filesystem::path& targetPath)
{
	// First we walk through the path elements starting from the root seeing if relativeTo and targetPath
	// match up.  As soon as they stop matching, we stop and create a new path that starts with ".." for
	// every unmatched directory left in relativeTo, and then append to that the remainder of the targetPath.

	ASSERT(relativeTo.has_root_path());
	ASSERT(targetPath.has_root_path());

	boost::filesystem::path pathA = relativeTo.branch_path();
	boost::filesystem::path pathB = targetPath.branch_path();

	boost::filesystem::path::const_iterator iterA = pathA.begin();
	boost::filesystem::path::const_iterator iterB = pathB.begin();

	for(; iterA != pathA.end() && iterB != pathB.end(); ++iterA, ++iterB)
	{
		if(*iterA != *iterB) break;
	}

	// Now start to construct our return path
	boost::filesystem::path retPath;
	for(; iterA != pathA.end(); ++iterA)
	{
		retPath /= "..";
	}

	for(; iterB != pathB.end(); ++iterB)
	{
		retPath /= *iterB;
	}

	retPath /= targetPath.leaf();

	return retPath;
}

