#ifndef QUADVIEWPANEL_H_
#define QUADVIEWPANEL_H_

#include "ProductionEditScene.h"

#include "wxWidgetGraphicsContext.h"
#include "EditCamera.h"

#include <Model.h>


#include <wx/wx.h>
#include <wx/glcanvas.h>

class EditPanel : public wxGLCanvas
{
	wxWidgetGraphicsContext* m_context;

	float m_fov;

	bool m_panning;
	bool m_rotating;

	bool m_movingSelected;
	bool m_scalingSelected;
	bool m_rotatingSelected;

	int m_prevMouseX;
	int m_prevMouseY;
	int m_curMouseX;
	int m_curMouseY;

	bool m_mouseMoved;

	ProductionEditScene& m_scene;

	EditCamera m_camera;
	EditCamera m_defaultCamera;

	float m_orthoZoom; // The zoom level, applies only to ortho mode

	ProductionEditScene::SceneChangedEvent::Connection* m_sceneChangedConnection;

	void TrySelectItem(int x, int y);

	Vector<4> m_markerPos;

public:
	enum ProjectionType
	{
		PT_PERSPECTIVE,
		PT_ORTHOGRAPHIC
	};

	typedef void ViewChangedSig(void);
	
private:
	ProjectionType m_projType;

	float m_orthoViewRad; // [Square]Radius of the orthographic view

	bool m_userControllableView;

	Event<ViewChangedSig> m_viewChanged;
	
public:
	EditPanel(ProductionEditScene& scene, wxWindow* parent, const wxPoint& pos, const wxSize& size);
	virtual ~EditPanel();

	Event<ViewChangedSig>::Port GetViewChangedPort() {return m_viewChanged.GetPort();}
	
	wxWidgetGraphicsContext& GetContext()
	{
		return *m_context;
	}

	void SetUserControllableView(bool enabled) {m_userControllableView = enabled;}
	bool GetUserControllableView() const {return m_userControllableView;}

	void SetViewRotation(const UnitQuaternion& rot);
	void SetDefaultViewRotation(const UnitQuaternion& rot);

	void OnMouseMoved(wxMouseEvent& event);
	void OnMouseDown(wxMouseEvent& event);
	void OnMouseReleased(wxMouseEvent& event);
	void OnRightDown(wxMouseEvent& event);
	void OnRightUp(wxMouseEvent& event);
	void OnMouseWheel(wxMouseEvent& event);
	void OnMouseDoubleClick(wxMouseEvent& event);
	void OnResize(wxSizeEvent& event);
	void OnMouseCaptureLost(wxMouseCaptureLostEvent& event);
	void OnPaint(wxPaintEvent& event);
	void OnEraseBackground(wxEraseEvent& event);

	void DoRedraw();

	void ResetPanelView();

	void SetProjectionType(ProjectionType projType) {m_projType = projType; DoRedraw();}
	ProjectionType GetProjectionType() const {return m_projType;}

	// Syncs the rendering context's width/height to the width/height of the window
	void RefreshContextSize();

	Matrix<4,4> GetProjectionMatrix() const;
	const EditCamera& GetCamera() const {return m_camera;}
	

	DECLARE_EVENT_TABLE()
};

class QuadViewPanel : public wxPanel
{
	EditPanel* m_editPanels[4];

	wxBoxSizer* m_mainSizer;
	wxPanel* m_editPanelHolder;
	wxGridSizer* m_editsGrid;

	ProductionEditScene& m_scene;

public:
	QuadViewPanel(ProductionEditScene& scene, wxWindow* parent);
	virtual ~QuadViewPanel();

	EditPanel& GetPanel(int panelIndex) {return *m_editPanels[panelIndex];}
	
	void OnRender(wxPaintEvent& event);
	void OnClose(wxCloseEvent& event);
	void OnActivate(wxActivateEvent& event);

	DECLARE_EVENT_TABLE()
};

#endif
