#ifndef WXWIDGETGRAPHICSCONTEXT_H_
#define WXWIDGETGRAPHICSCONTEXT_H_

#include "GraphicsContext.h"
#include "ThreadPrimitives.h"
#include "CommandQueue.h"
#include "OpenGLRenderContext.h"

#include <wx/wx.h>

#include <set>

#include <boost/function.hpp>

class wxGLCanvas;
class wxGLContext;

class wxWidgetGraphicsContext : public GraphicsContext
{
	OpenGLRenderContext m_glContext;

	wxGLCanvas& m_canvas;

	wxGLContext* m_wxContext;

	Mutex m_initMutex;
	Condition m_initCond;

	int m_width;
	int m_height;

	float m_near;
	float m_far;

	Thread* m_thread;
	CommandQueue* m_commandQueue;
	
	void Run();


	struct RenderFrameCommand : public CommandQueue::Command
	{
		wxWidgetGraphicsContext& handler;

		GraphicsFrame& frame;
		Matrix<4,4> proj;
		GraphicsContext::RenderOptions options;
		GraphicsFrame::Freeze freeze;

		RenderFrameCommand(wxWidgetGraphicsContext& pHandler, GraphicsFrame& pFrame, const GraphicsContext::RenderOptions& pOptions, const Matrix<4,4>& pProj, GraphicsFrame::Freeze& pFreeze)
			: handler(pHandler)
			, frame(pFrame)
			, proj(pProj)
			, options(pOptions)
			, freeze(pFreeze)
		{};

		virtual void ProcessCommand() {handler.ProcessCommand(*this);}
	};
	void ProcessCommand(RenderFrameCommand& cmd);

	struct ResizeViewportCommand : public CommandQueue::Command
	{
		wxWidgetGraphicsContext& handler;

		int width;
		int height;

		ResizeViewportCommand(wxWidgetGraphicsContext& pHandler, int pWidth, int pHeight)
			: handler(pHandler)
			, width(pWidth), height(pHeight)
		{};

		virtual void ProcessCommand() {handler.ProcessCommand(*this);}
	};
	void ProcessCommand(ResizeViewportCommand& cmd);

public:
	wxWidgetGraphicsContext(wxGLCanvas& canvas);
	~wxWidgetGraphicsContext();

	virtual void ResizeScreen(int width, int height);

	virtual void RenderGraphicsFrame(GraphicsFrame& frame, const GraphicsContext::RenderOptions& options);
	virtual void FlushWait();

	void SetNearPlane(float n) {m_near = n;}
	float GetNearPlane() {return m_near;}

	void SetFarPlane(float f) {m_far = f;}
	float GetFarPlane() {return m_far;}

	virtual int GetWidth();
	virtual int GetHeight();
};

#endif
