#include "wxWidgetKeyboardInput.h"

BEGIN_EVENT_TABLE(wxWidgetKeyboardInput::EventHandler, wxEvtHandler)
	EVT_KEY_DOWN(wxWidgetKeyboardInput::EventHandler::OnKeyDown)
	EVT_KEY_UP(wxWidgetKeyboardInput::EventHandler::OnKeyUp)
END_EVENT_TABLE()

void wxWidgetKeyboardInput::EventHandler::OnKeyDown(wxKeyEvent& event)
{
	event.Skip();

	m_parent.OnKeyDown(event);
}

void wxWidgetKeyboardInput::EventHandler::OnKeyUp(wxKeyEvent& event)
{
	event.Skip();

	m_parent.OnKeyUp(event);
}

static KeyboardInput::KeyType ScanCodeToKeyType(int key)
{
	switch(key)
	{
		case WXK_BACK: return KeyboardInput::KT_BACKSPACE;
		case WXK_ESCAPE: return KeyboardInput::KT_ESC;

		case WXK_F1: return KeyboardInput::KT_F1;
		case WXK_F2: return KeyboardInput::KT_F2;
		case WXK_F3: return KeyboardInput::KT_F3;
		case WXK_F4: return KeyboardInput::KT_F4;
		case WXK_F5: return KeyboardInput::KT_F5;
		case WXK_F6: return KeyboardInput::KT_F6;
		case WXK_F7: return KeyboardInput::KT_F7;
		case WXK_F8: return KeyboardInput::KT_F8;
		case WXK_F9: return KeyboardInput::KT_F9;
		case WXK_F10: return KeyboardInput::KT_F10;
		case WXK_F11: return KeyboardInput::KT_F11;
		case WXK_F12: return KeyboardInput::KT_F12;
		case WXK_PAUSE: return KeyboardInput::KT_PAUSE;
		
		case '~': return KeyboardInput::KT_TILDE;
		case '1': return KeyboardInput::KT_1;
		case '2': return KeyboardInput::KT_2;
		case '3': return KeyboardInput::KT_3;
		case '4': return KeyboardInput::KT_4;
		case '5': return KeyboardInput::KT_5;
		case '6': return KeyboardInput::KT_6;
		case '7': return KeyboardInput::KT_7;
		case '8': return KeyboardInput::KT_8;
		case '9': return KeyboardInput::KT_9;
		case '0': return KeyboardInput::KT_0;
		case '-': return KeyboardInput::KT_MINUS;
		case '+': return KeyboardInput::KT_EQUAL;
		
		case WXK_TAB: return KeyboardInput::KT_TAB;
		case 'Q': return KeyboardInput::KT_Q;
		case 'W': return KeyboardInput::KT_W;
		case 'E': return KeyboardInput::KT_E;
		case 'R': return KeyboardInput::KT_R;
		case 'T': return KeyboardInput::KT_T;
		case 'Y': return KeyboardInput::KT_Y;
		case 'U': return KeyboardInput::KT_U;
		case 'I': return KeyboardInput::KT_I;
		case 'O': return KeyboardInput::KT_O;
		case 'P': return KeyboardInput::KT_P;
		case '[': return KeyboardInput::KT_SQUARE_BRACKET_L;
		case ']': return KeyboardInput::KT_SQUARE_BRACKET_R;
		case '\\': return KeyboardInput::KT_BACKSLASH;
		
		case WXK_CAPITAL: return KeyboardInput::KT_CAPSLOCK;
		case 'A': return KeyboardInput::KT_A;
		case 'S': return KeyboardInput::KT_S;
		case 'D': return KeyboardInput::KT_D;
		case 'F': return KeyboardInput::KT_F;
		case 'G': return KeyboardInput::KT_G;
		case 'H': return KeyboardInput::KT_H;
		case 'J': return KeyboardInput::KT_J;
		case 'K': return KeyboardInput::KT_K;
		case 'L': return KeyboardInput::KT_L;
		case ';': return KeyboardInput::KT_SEMICOLON;
		case '\'': return KeyboardInput::KT_APOSTROPHE;
		case WXK_RETURN: return KeyboardInput::KT_ENTER;

		case WXK_SHIFT: return KeyboardInput::KT_SHIFT;
		case 'Z': return KeyboardInput::KT_Z;
		case 'X': return KeyboardInput::KT_X;
		case 'C': return KeyboardInput::KT_C;
		case 'V': return KeyboardInput::KT_V;
		case 'B': return KeyboardInput::KT_B;
		case 'N': return KeyboardInput::KT_N;
		case 'M': return KeyboardInput::KT_M;
		case ',': return KeyboardInput::KT_COMMA;
		case '.': return KeyboardInput::KT_PERIOD;
		case '/': return KeyboardInput::KT_SLASH;

		case WXK_CONTROL: return KeyboardInput::KT_CTRL;
		case WXK_SPACE: return KeyboardInput::KT_SPACE;
		
		case WXK_LEFT: return KeyboardInput::KT_LEFT;
		case WXK_RIGHT: return KeyboardInput::KT_RIGHT;
		case WXK_UP: return KeyboardInput::KT_UP;
		case WXK_DOWN: return KeyboardInput::KT_DOWN;

		case WXK_INSERT: return KeyboardInput::KT_INSERT;
		case WXK_DELETE: return KeyboardInput::KT_DELETE;
		case WXK_HOME: return KeyboardInput::KT_HOME;
		case WXK_END: return KeyboardInput::KT_END;
		case WXK_PRIOR: return KeyboardInput::KT_PG_UP;
		case WXK_NEXT: return KeyboardInput::KT_PG_DOWN;

		default: return KeyboardInput::KT_INVALID;
	};
}

void wxWidgetKeyboardInput::OnKeyDown(wxKeyEvent& event)
{
	// Make sure the key's not already pressed
	if(m_keysDown.find(event.GetKeyCode()) == m_keysDown.end())
	{
		m_keysDown.insert(event.GetKeyCode());

		// Translate to our key codes
		KeyEvent evt;
		evt.key = ScanCodeToKeyType(event.GetKeyCode());
		evt.pressed = true;

		PostKeyPress(evt);
	}
}

void wxWidgetKeyboardInput::OnKeyUp(wxKeyEvent& event)
{
	m_keysDown.erase(event.GetKeyCode());

	// Translate to our key codes
	KeyEvent evt;
	evt.key = ScanCodeToKeyType(event.GetKeyCode());
	evt.pressed = false;

	PostKeyPress(evt);
}

wxWidgetKeyboardInput::wxWidgetKeyboardInput(wxEvtHandler& handler)
{
	new(m_handler) EventHandler(*this);

	// Find the end of the event handling chain
	wxEvtHandler* nextHandler = handler.GetNextHandler();

	// Add our handler to the handling chain
	handler.SetNextHandler(m_handler.GetPtr());
	m_handler->SetPreviousHandler(&handler);
	m_handler->SetNextHandler(nextHandler);
}

