#include "wxWidgetMouseInput.h"

#include <iostream>

BEGIN_EVENT_TABLE(wxWidgetMouseInput::EventHandler, wxEvtHandler)
	EVT_MOTION(wxWidgetMouseInput::EventHandler::OnMouseMoved)
	EVT_LEFT_DOWN(wxWidgetMouseInput::EventHandler::OnLeftDown)
	EVT_LEFT_UP(wxWidgetMouseInput::EventHandler::OnLeftUp)
	EVT_RIGHT_DOWN(wxWidgetMouseInput::EventHandler::OnRightDown)
	EVT_RIGHT_UP(wxWidgetMouseInput::EventHandler::OnRightUp)
END_EVENT_TABLE()

void wxWidgetMouseInput::EventHandler::OnMouseMoved(wxMouseEvent& event)
{
	m_parent.OnMouseMoved(event);

	event.Skip();
}

void wxWidgetMouseInput::EventHandler::OnLeftDown(wxMouseEvent& event)
{
	m_parent.OnLeftDown(event);

	event.Skip();
}

void wxWidgetMouseInput::EventHandler::OnLeftUp(wxMouseEvent& event)
{
	m_parent.OnLeftUp(event);

	event.Skip();
}

void wxWidgetMouseInput::EventHandler::OnRightDown(wxMouseEvent& event)
{
	m_parent.OnRightDown(event);

	event.Skip();
}

void wxWidgetMouseInput::EventHandler::OnRightUp(wxMouseEvent& event)
{
	m_parent.OnRightUp(event);

	event.Skip();
}


wxWidgetMouseInput::wxWidgetMouseInput(wxEvtHandler& handler)
: m_lastX(-1)
, m_lastY(-1)
{
	new(m_handler) EventHandler(*this);

	// Find the end of the event handling chain
	wxEvtHandler* nextHandler = handler.GetNextHandler();

	// Add our handler to the handling chain
	handler.SetNextHandler(m_handler.GetPtr());
	m_handler->SetPreviousHandler(&handler);
	m_handler->SetNextHandler(nextHandler);
}




void wxWidgetMouseInput::OnMouseMoved(wxMouseEvent& event)
{
	if(m_lastX != -1)
	{
		MoveEvent evt;
		evt.dx = event.GetX() - m_lastX;
		evt.dy = event.GetY() - m_lastY;

		PostMoveEvent(evt);
	}

	m_lastX = event.GetX();
	m_lastY = event.GetY();
}

void wxWidgetMouseInput::OnLeftDown(wxMouseEvent& event)
{
	ButtonEvent evt;
	evt.button = 0;
	evt.pressed = true;

	PostButtonEvent(evt);
}

void wxWidgetMouseInput::OnLeftUp(wxMouseEvent& event)
{
	ButtonEvent evt;
	evt.button = 0;
	evt.pressed = false;

	PostButtonEvent(evt);
}

void wxWidgetMouseInput::OnRightDown(wxMouseEvent& event)
{
	ButtonEvent evt;
	evt.button = 2;
	evt.pressed = true;

	PostButtonEvent(evt);
}

void wxWidgetMouseInput::OnRightUp(wxMouseEvent& event)
{
	ButtonEvent evt;
	evt.button = 2;
	evt.pressed = false;

	PostButtonEvent(evt);
}
